/*
 File:		TaskWrapper.m

 Description: 	This is the implementation of a generalized process handling class that that makes asynchronous interaction with an NSTask easier.  Feel free to make use of this code in your own applications.  TaskWrapper objects are one-shot (since NSTask is one-shot); if you need to run a task more than once, destroy/create new TaskWrapper objects.

 Author:		EP & MCF

 Copyright: 	 Copyright 2001 Apple Computer, Inc. All rights reserved.

 */



#import "TaskWrapper.h"

@implementation TaskWrapper

- (id)initWithDelegate:(id)inDelegate arguments:(NSArray *)args
{
    self = [self init];

    if (self)	{
		_delegate = inDelegate;
		_ownDelegate = NO	;
		[inDelegate setTask:self]	;
		_arguments = [args retain];
    }
	
    return self;
}

- (void)retainDelegate	;
{
		_ownDelegate = YES	;
		[_delegate retain]	;
}

- (NSString *)description
{
		return [NSString stringWithFormat:@"TaskWrapper <0x%X> {\n\t_delegate: %@\n\t_ownDelegate %d\n\t_arguments %@\n\t_task %@ }",self,_delegate,_ownDelegate,_arguments,_task]	;
}

- (void)dealloc
{
		if (_delegate)	{
			[self stopProcess];
			[[NSNotificationCenter defaultCenter] removeObserver:_delegate name:NSTaskDidTerminateNotification object:_task];
			if (_ownDelegate)
				[_delegate release]	;
		}
	
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object: [[_task standardOutput] fileHandleForReading]];
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object: [[_task standardError] fileHandleForReading]];
		[_arguments release];
		[_task release];
		[super dealloc];
}

- (void) startProcess
{
		[_delegate processStarted];

		_task = [[NSTask alloc] init];

		[_task setStandardOutput: [NSPipe pipe]];
		[_task setStandardError: [NSPipe pipe]];
		[_task setLaunchPath: [_arguments objectAtIndex:0]];
		[_task setArguments: [_arguments subarrayWithRange: NSMakeRange (1, ([_arguments count] - 1))]];

		[[NSNotificationCenter defaultCenter] addObserver:self 
			selector:@selector(getData:) 
			name: NSFileHandleReadCompletionNotification 
			object: [[_task standardOutput] fileHandleForReading]];

		[[NSNotificationCenter defaultCenter] addObserver:self 
			selector:@selector(getError:) 
			name: NSFileHandleReadCompletionNotification 
			object: [[_task standardError] fileHandleForReading]];
 		
		[[NSNotificationCenter defaultCenter] addObserver:_delegate selector:@selector(taskCompleted:) name:NSTaskDidTerminateNotification object:_task];

		[[[_task standardOutput] fileHandleForReading] readInBackgroundAndNotify];

	    [_task launch];    
}

- (void) stopProcess
{
		[_delegate processFinished];    
		_delegate = nil;
		[_task terminate];
	NSLog(@"terminationStatus %d",[_task terminationStatus])	;
}

- (void) getData: (NSNotification *)aNotification
{
    NSData *data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];

		if ([data length])
		{
			[_delegate appendOutput: data];
		} 
		else if (![_task isRunning])	{
			[self stopProcess];
			return ;
		}
    
		if ([_task isRunning])
		{
			[[aNotification object] readInBackgroundAndNotify];
		}
		else
		{
			[self stopProcess];
		}    
}

- (void) getError: (NSNotification *)aNotification
{
    NSData *data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];

		if ([data length])
		{
			[_delegate appendError: [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease]];
		}
	    
		if (_task)
		{
			[[aNotification object] readInBackgroundAndNotify];
		}
}

@end

